
public class Napoleon
{
	// instance variables	
	private Lista[] lista;  // 8 
	private Lista[] pila;  //  4
    private Mazzo mazzo;
    private Lista listaOrigine, listaDestinazione; // to permit UNDO
    // counters
    private int numeroPartiteGiocate;
    private int numeroPartiteVinte;
    

	public Napoleon()
	{
        mazzo = new MazzoPokerIncompleto();
		// 4 stacks
		pila = new Lista[4];
		for(int i = 0; i < 4; i++)
		{
		    pila[i] = new Lista(10, true);  // true ==> stack
		}
		lista = new Lista[8];
		for(int i = 0; i < 8; i++)
		{
		    lista[i] = new Lista(15, false);  // false ==> side by side
		}
		// to manage UNDO
        listaOrigine = null;
        listaDestinazione = null;
        // statistics initialization
        numeroPartiteGiocate = 1;
        numeroPartiteVinte = 0;

	}


    public void preparaNuovaPartita()
    {
    	for(int i = 0; i < 4; i++)
		{
		    pila[i].svuota();
		}
		for(int i = 0; i < 8; i++)
		{
		    lista[i].svuota();
		}
		// to manage UNDO
        listaOrigine = null;
        listaDestinazione = null;
        // statistics update
        numeroPartiteGiocate++;
    }
    
    
    public void mescolaEDaiCarte()
    {
        mazzo.mescola();
        int i = 0;   // card index
        int j = 0;   // stack index 
        int k = 0;   // deck index
        int n = mazzo.numeroCarte();
        boolean finito = false;
        while (i < n)
        {
            Carta carta = mazzo.prendiCarta(i);
            if ((carta.leggiValore() == 1) && (pila[j].leggiNumeroCarte() == 0))
            {  pila[j].aggiungi(carta); }
            else
            {  lista[k].aggiungi(carta); }    
            i++; 
            if ((k % 2)  == 1) {j = (j+1) %4;} // incremento j solo una volta si e una no!
            k = (k + 1) % 8;
        }          
    }
    
    
    public boolean undoSpostamento()
    {
        if (( listaOrigine != null ) && ( listaDestinazione != null))
        {
           Carta carta = listaDestinazione.leggiTop();
           listaDestinazione.togliCarta();
           listaOrigine.aggiungi(carta);
           listaOrigine = null;
           listaDestinazione = null;
           return true;
        }   
        else
        {
            return false;
        }
    }    
    
    
    public boolean sposta(Lista origine, Lista destinazione)
    {  // returns true if the move is valid, false otherwise
       int n = origine.leggiNumeroCarte();
       if (n == 0) 
          { return false; }
       else if (destinazione.pila())
          {
              Carta carta = origine.leggiCarta(n-1);
              int n2 = destinazione.leggiNumeroCarte();
              if (n2 > 0) 
              {
                   Carta carta2 = destinazione.leggiTop();
                   if (!carta.leggiSeme().equals(carta2.leggiSeme()))
                   {  return false; }
                   else if (Math.abs(carta.leggiValore() - carta2.leggiValore()) != 1)
                   { return false; }
                   else
                   {
                      // to manage UNDO
                      listaOrigine = origine;
                      listaDestinazione = destinazione;
                      //
                      origine.togliCarta();
                      destinazione.aggiungi(carta);
                      return true;
                   }
               }
               else  // n2 == 0
               {
                   if (carta.leggiValore() == 1)
                   {
                       // to manage UNDO
                       listaOrigine = origine;
                       listaDestinazione = destinazione;
                       //
                       origine.togliCarta();
                       destinazione.aggiungi(carta);
                       return true;
                    }
                    else
                    { return false; }
                }                    
          }   
       else       
          {       
            Carta carta = origine.leggiCarta(n-1);
            int n2 = destinazione.leggiNumeroCarte();
            if (n2 > 0) 
               {
                   Carta carta2 = destinazione.leggiCarta(n2-1);
                   if (!carta.leggiSeme().equals(carta2.leggiSeme()))
                   {  return false; }
                   else if (Math.abs(carta.leggiValore() - carta2.leggiValore()) != 1)
                   { return false; }
                   else
                   {
                     // per l'UNDO
                     listaOrigine = origine;
                     listaDestinazione = destinazione;
                     //  
                     origine.togliCarta();
                     destinazione.aggiungi(carta);
                     return true;
                   }
                }
             else  // n2 == 0
             {
                 // to manage UNDO
                 listaOrigine = origine;
                 listaDestinazione = destinazione;
                 //
                 origine.togliCarta();
                 destinazione.aggiungi(carta);
                 return true;
             }                    
          }   
    }

     
    public boolean haiVinto()
    {
        boolean vinto = true;
        for(int i = 0; i < 4; i++)
        {
            int n = pila[i].leggiNumeroCarte();
            if (n < 10)
            {  vinto = false; }            
        }
        if (vinto)
        { numeroPartiteVinte++; }
        return vinto;
    }


    public Lista leggiPila(int posizione)
    {
        return pila[posizione];  //  0 to 3
    }
    
    public Lista leggiLista(int posizione)
    {
        return lista[posizione];  //  0 to 7
    }
    
    public String leggiStatistica()
    {
        int percentuale;
        if (numeroPartiteGiocate > 0)
        { percentuale = numeroPartiteVinte * 100 / numeroPartiteGiocate; }
        else
        { percentuale = 0; }
        String s;
        if (numeroPartiteVinte == 1) { s = " game ";} else { s = " games ";}
        return "You have won " + numeroPartiteVinte + s + " out of " + numeroPartiteGiocate + " (" +
               + percentuale + "%)";
    }
    
}
